@@ -53,6 +53,18 @@ class AnswerBox(object):
             <p style="display:inline">Integer 2:<textline correct_answer="9" inline="1"/></p>
         </customresponse>
 
+        -----------------------------------------------------------------------------
+        <abox type="jsinput" expect="(3 * 5) / (2 + 3)" cfn="eq" gradefn="gradefn" height="500" 
+               get_statefn="getstate" set_statefn="setstate" html_file="/static/jsinput.html"/> 
+        
+        <customresponse cfn="eq" expect="(3 * 5) / (2 + 3)">
+            <jsinput gradefn="gradefn" 
+                height="500"
+                get_statefn="getstate"
+                set_statefn="setstate"
+                html_file="/static/jsinput.html"/>
+        </customresponse>
+        
         -----------------------------------------------------------------------------
         <abox type="numerical" expect="3.141" tolerance="5%" />
         
@@ -130,7 +142,8 @@ class AnswerBox(object):
                           'shortanswer' : 'shortanswerresponse',
                           'string': 'stringresponse',
                           'symbolic': 'symbolicresponse',
-                          'image': 'imageresponse'
+                          'image': 'imageresponse',
+                          'jsinput': 'customresponse_jsinput',
                           }
 
         if 'type' in abargs and abargs['type'] in type2response:
@@ -213,6 +226,7 @@ class AnswerBox(object):
             abxml.set('answer',self.stripquotes(abargs['expect']))
             self.copy_attrib(abargs,'inline',tl)
             self.copy_attrib(abargs,'inline',abxml)
+            abxml.set('type','')
 
         elif abtype=='customresponse':
             self.require_args(['expect','cfn'])
@@ -242,6 +256,8 @@ class AnswerBox(object):
                 tl.set('correct_answer',ans)
                 self.copy_attrib(abargs,'inline',tl)
                 self.copy_attrib(abargs,'math',tl)
+                self.copy_attrib(abargs,'preprocessorClassName', tl)
+                self.copy_attrib(abargs,'preprocessorSrc', tl)
                 if prompt:
                     elem = etree.Element('p')
                     if 'inline' in abargs:
@@ -255,7 +271,21 @@ class AnswerBox(object):
                 abxml.append(elem)
                 cnt += 1
                     
-            
+        elif abtype=='customresponse_jsinput':
+            abxml.tag = 'customresponse'
+            self.require_args(['expect','cfn'])
+            abxml.set('cfn',self.stripquotes(abargs['cfn']))
+            self.copy_attrib(abargs,'expect',abxml)
+            self.copy_attrib(abargs,'options',abxml)
+            if abxml.get('options',''):
+                abxml.set('cfn_extra_args','options')	# tells sandbox to include 'options' in cfn call arguments
+
+            js = etree.Element('jsinput')
+            jsattribs = ['width', 'height', 'gradefn', 'get_statefn', 'set_statefn', 'html_file']
+            for jsa in jsattribs:
+                self.copy_attrib(abargs, jsa, js)
+            abxml.append(js)
+                    
         elif abtype=='externalresponse' or abtype== 'coderesponse':
             if 'url' in abargs:
                 self.copy_attrib(abargs,'url',abxml)
@@ -305,6 +335,8 @@ class AnswerBox(object):
             self.copy_attrib(abargs,'size',tl)
             self.copy_attrib(abargs,'inline',tl)
             self.copy_attrib(abargs,'math',tl)
+            self.copy_attrib(abargs,'preprocessorClassName', tl)
+            self.copy_attrib(abargs,'preprocessorSrc', tl)
             abxml.append(tl)
             answer = self.stripquotes(abargs['expect'])
             abxml.set('answer',answer)
@@ -320,6 +352,8 @@ class AnswerBox(object):
             tl = etree.Element('textline')
             self.copy_attrib(abargs,'inline',tl)
             self.copy_attrib(abargs,'size',tl)
+            self.copy_attrib(abargs,'preprocessorClassName', tl)
+            self.copy_attrib(abargs,'preprocessorSrc', tl)
             abxml.append(tl)
             self.copy_attrib(abargs,'inline',abxml)
             if 'correct_answer' in abargs:
